import { ApolloError, FetchResult, useMutation, useQuery } from '@apollo/client'
import { Dispatch, SetStateAction, useEffect, useState } from 'react'
import { useTranslation } from 'react-i18next'

import { isMaliciousUrlError, isNotFoundError } from '@holi/core/errors/helpers'
import { useErrorHandling } from '@holi/core/errors/hooks'
import useRouting from '@holi/core/navigation/hooks/useRouting'
import { PostById, PostByIdResponse } from '@holi/core/screens/posts/queries'
import { PostImage } from '@holi/core/screens/posts/types'
import { useSpace } from '@holi/core/screens/spaces/hooks/useSpace'
import { Space, VisibilityFilter, VisibilityType } from '@holi/core/screens/spaces/types'
import {
  CreateSpacePostMutationResponse,
  UpdateSpacePostMutationResponse,
  createSpacePostMutation,
  updateSpacePostMutation,
} from '@holi/core/screens/spaces/updates/mutations'
import { spacePostsQuery } from '@holi/core/screens/spaces/updates/queries'
import { HoliToastType, useToast } from '@holi/ui/components/molecules/HoliToastProvider'
import { HoliImageInfo } from '@holi/ui/components/organisms/HoliImagePicker/types'

export interface UseCreateOrUpdateSpacePost {
  // Status
  loading: boolean
  error?: ApolloError
  isUpdate: boolean
  // meta
  space?: Space
  // Form State
  title: string
  content: string
  image?: HoliImageInfo | null
  currentImage?: PostImage
  visibility?: VisibilityType
  setTitle: Dispatch<SetStateAction<string>>
  setContent: Dispatch<SetStateAction<string>>
  setImage: Dispatch<SetStateAction<HoliImageInfo | undefined | null>>
  setVisibility: Dispatch<SetStateAction<VisibilityType>>
  // interface
  saveSpacePost: () => Promise<FetchResult>
}

export const useCreateOrUpdateSpacePost = (spaceIdOrName?: string, postId?: string): UseCreateOrUpdateSpacePost => {
  const { displayError } = useErrorHandling()
  const { navigateBack, replaceRoute } = useRouting()
  const { openToast } = useToast()
  const { t } = useTranslation()

  //input fields
  const [title, setTitle] = useState<string>('')
  const [content, setContent] = useState<string>('')
  const [image, setImage] = useState<HoliImageInfo | undefined | null>() // null is to remove the image
  const [currentImage, setCurrentImage] = useState<PostImage>()
  const [visibility, setVisibility] = useState<VisibilityType>(VisibilityType.MEMBERS)

  // data fetch
  const { loading: spaceQueryLoading, error: spaceQueryError, space } = useSpace(spaceIdOrName)

  const {
    data: postQueryData,
    error: postQueryError,
    loading: postQueryLoading,
  } = useQuery<PostByIdResponse>(PostById, {
    variables: { id: postId },
    skip: !postId,
    onError: (error) => {
      if (isNotFoundError(error)) {
        return replaceRoute('/404')
      }
      navigateBack()
      displayError(error, 'Failed to load space post', {
        location: 'useCreateOrUpdateSpacePost.PostById',
        spaceIdOrName,
        postId,
      })
    },
  })
  const postToUpdate = postQueryData?.postById

  useEffect(() => {
    if (postToUpdate) {
      postToUpdate.title && setTitle(postToUpdate.title)
      postToUpdate.text && setContent(postToUpdate.text)
      postToUpdate.image ?? setImage(postToUpdate.image)
      setCurrentImage(postToUpdate.image)
      setVisibility(postToUpdate.isPublic ? VisibilityType.PUBLIC : VisibilityType.MEMBERS)
    }
  }, [postToUpdate])

  // create & update
  const [createSpacePost, { loading: loadingCreateMutation }] = useMutation<CreateSpacePostMutationResponse>(
    createSpacePostMutation,
    {
      onCompleted: () => {
        navigateBack()
      },
      onError: (error) => {
        if (isMaliciousUrlError(error)) {
          openToast(t('global.maliciousUrl.error'), HoliToastType.ERROR)
        } else {
          displayError(error, 'Failed to create space post', {
            location: 'useCreateOrUpdateSpacePost.createSpacePost',
            spaceIdOrName,
          })
        }
      },
      // refetch first page of both the current visibility filter and the ALL filter
      refetchQueries: [
        {
          query: spacePostsQuery,
          variables: { idOrName: spaceIdOrName, offset: 0, limit: 10, visibility: visibility },
        },
        {
          query: spacePostsQuery,
          variables: { idOrName: spaceIdOrName, offset: 0, limit: 10, visibility: VisibilityFilter.ALL },
        },
      ],
    }
  )

  const [updateSpacePost, { loading: loadingUpdateMutation }] = useMutation<UpdateSpacePostMutationResponse>(
    updateSpacePostMutation,
    {
      onCompleted: () => navigateBack(),
      onError: (error) => {
        if (isMaliciousUrlError(error)) {
          openToast(t('global.maliciousUrl.error'), HoliToastType.ERROR)
        } else {
          displayError(error, 'Failed to update space post', {
            location: 'useCreateOrUpdateSpacePost.updateSpacePost',
            spaceIdOrName,
            postId,
          })
        }
      },
      refetchQueries: [spacePostsQuery],
    }
  )

  const saveSpacePost = () => {
    const input = {
      title: title.trim() && title.length > 0 ? title : undefined,
      text: content.trim(),
      image: image === null ? null : image?.blob, // null is to remove the image
      visibility: visibility || VisibilityType.MEMBERS,
    }

    if (postId) {
      return updateSpacePost({ variables: { input: { postId, ...input } } })
    } else {
      return createSpacePost({ variables: { input: { spaceId: space?.id, ...input } } })
    }
  }

  return {
    loading: spaceQueryLoading || postQueryLoading || loadingCreateMutation || loadingUpdateMutation,
    error: spaceQueryError || postQueryError,
    isUpdate: !!postId,
    space,
    title,
    content,
    image,
    currentImage,
    visibility,
    setTitle,
    setContent,
    setImage,
    setVisibility,
    saveSpacePost,
  }
}

export default useCreateOrUpdateSpacePost
